/** Copyright (C) 2017 by Bundesanstalt für Gewässerkunde
 * Software engineering by
 *  Björnsen Beratende Ingenieure GmbH
 *  Dr. Schumacher Ingenieurbüro für Wasser und Umwelt
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */

package org.dive4elements.river.artifacts.bundu.bezugswst;

import java.util.Date;
import java.util.Map.Entry;
import java.util.NavigableMap;
import java.util.TreeMap;

import org.dive4elements.artifacts.CallContext;
import org.dive4elements.river.artifacts.access.BedQualityAccess;
import org.dive4elements.river.artifacts.bundu.BUNDUArtifact;
import org.dive4elements.river.artifacts.math.Linear;
import org.dive4elements.river.artifacts.model.Calculation;
import org.dive4elements.river.artifacts.model.Calculation.Problem;
import org.dive4elements.river.artifacts.model.CalculationResult;
import org.dive4elements.river.artifacts.model.minfo.BedQualityCalculation;
import org.dive4elements.river.artifacts.model.minfo.BedQualityResult;
import org.dive4elements.river.artifacts.model.minfo.BedQualityResultValue;
import org.dive4elements.river.model.River;

/**
 * Calculator for bed quality parameters in a km range and time period, wrapping the minfo BedQualityCalculation
 *
 * @author Matthias Schäfer
 *
 */
public class BedQualityCalculator {

    private final CallContext context;

    private final BUNDUArtifact bundu;

    private final NavigableMap<Double, Double> densities;

    public BedQualityCalculator(final CallContext context, final BUNDUArtifact bundu) {
        this.context = context;
        this.bundu = bundu;
        this.densities = new TreeMap<>();
    }


    /**
     * Calculates the river bed sublayer densities for an array of kms and a time period of measurements
     */
    public void execute(final Calculation problems, final River river, final double[] kms, final Date startDay, final Date endDay) {
        final BedQualityCalculation bqCalc = new BedQualityCalculation();
        final BedQualityAccess access = createBqAccess(kms[0], kms[kms.length - 1], startDay, endDay);
        final CalculationResult bqCalcResult = bqCalc.calculate(access);
        if (bqCalcResult.getReport().getProblems() != null) {
            for (final Problem problem : bqCalcResult.getReport().getProblems())
                problems.addProblem(problem);
        }
        final BedQualityResult[] results = (BedQualityResult[]) bqCalcResult.getData();
        final BedQualityResult result = results[0];
        final BedQualityResultValue bqResValue = result.getValue("density", "sublayer");
        final double[][] kmdensities = bqResValue.getDataInterpolated(kms);
        this.densities.clear();
        for (int i = 0; i <= kmdensities[0].length - 1; i++)
            this.densities.put(Double.valueOf(kmdensities[0][i]), Double.valueOf(kmdensities[1][i]));
    }

    /**
     * Searches the density of a station in the active calculation result
     */
    public double getDensity(final double station) {
        return interpolateDensity(station);
    }

    /**
     * Searches and interpolates a density value for a km
     */
    private double interpolateDensity(final double km) {

        if (this.densities.containsKey(km)) {
            final Double value = this.densities.get(km);
            return (value == null) ? Double.NaN : value.doubleValue();
        }

        final Entry<Double, Double> floorEntry = this.densities.floorEntry(km);
        final Entry<Double, Double> ceilingEntry = this.densities.ceilingEntry(km);

        if ((floorEntry == null) || (ceilingEntry == null))
            return Double.NaN;

        final double floorKm = floorEntry.getKey().doubleValue();
        final double ceilKm = ceilingEntry.getKey().doubleValue();

        // report once if the interpolation distance exceeds 1000m
        // if ((Math.abs(floorKm - ceilKm) > MAX_DISTANCE_KM) && (this.problems != null)) {
        // this.problems.addProblem(km, "linearInterpolator.maxdistance", MAX_DISTANCE_KM * 1000);
        // this.problems = null;
        // return Double.NaN;
        // }

        final Double floorHeight = floorEntry.getValue();
        final Double ceilingHeight = ceilingEntry.getValue();

        if (floorHeight == null || ceilingHeight == null)
            return Double.NaN;

        return Linear.linear(km, floorKm, ceilKm, floorHeight, ceilingHeight);
    }

    /**
     * Creates an access object for the bed quality calculation
     */
    private BedQualityAccess createBqAccess(final double fromKm, final double toKm, final Date startDay, final Date endDay) {
        final BunduMinfoArtifactWrapper minfo = new BunduMinfoArtifactWrapper(this.bundu, startDay, endDay);
        return new BedQualityAccess(minfo, this.context);
    }
}